"""
Public interface to use powerpoint presentations as export template.
"""
from pptx import Presentation

from .engines import PythonEngine
from .slides import SlideContainer

__all__ = ['Template']


class TemplateIterator:
    """
    Iterater that knows how to deal with newly inserted slides.

    This is private API.
    """

    def __init__(self, slides):
        self.slides = slides
        self.current = 0

    def __next__(self):
        """
        Return the next slide in the slideset, which may have been inserted.
        """
        if self.current >= len(self.slides):
            raise StopIteration
        slide = self.slides[self.current]
        self.current += 1
        return slide


class Template:
    """
    A powerpoint presentation that serves as a template.

    :param filepath: path to the powerpoint file on disk or filelike object.
    """

    def __init__(self, pptx):
        self._presentation = Presentation(pptx)

    def __iter__(self):
        return TemplateIterator(self._presentation.slides)

    @property
    def layouts(self):
        """
        Returns the names of the slide layouts present in the template file.
        """
        return [layout.name for layout in self._presentation.slide_layouts]

    def render(self, context, engine=PythonEngine()):

        for slide in self:
            slide = SlideContainer(slide, self._presentation)
            slide.render(engine, context)

    def save_to(self, outfile):
        self._presentation.save(outfile)



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='burocracy',
    version='0.2',
    license='MIT',

    install_requires=[
        'pypandoc',
        'python-docx',
        'python-pptx>=0.6.2',
    ],
    include_package_data=True,
    packages=find_packages(exclude=["tests"]),

    setup_requires=['pytest-runner'],
    tests_require=[
        'PyPDF2',
        'pytest-cov',
    ],

    description='Templating and pdf generation for docx/pptx files',
    long_description=README,
    author='Maykin Media, Robin Ramael, Sergei Maertens',
    author_email='robin.ramael@maykinmedia.nl, sergei@maykinmedia.nl',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



import os
import warnings

from pptx.enum.shapes import PP_PLACEHOLDER
from pptx.shapes.placeholder import BasePlaceholder

from .engines import BaseEngine

CONTEXT_KEY_FOR_PLACEHOLDER = 'PPT_CURRENT_PLACEHOLDER'

AlreadyRendered = object()


class AlreadyRenderedException(Exception):
    pass


class PlaceholderContainer:
    def __init__(self, placeholder: BasePlaceholder, fragment: str):
        self.placeholder = placeholder
        self.fragment = fragment

    def render(self, engine: BaseEngine, context: dict):
        context[CONTEXT_KEY_FOR_PLACEHOLDER] = self

        try:
            rendered = engine.render(self.fragment, context)
            if rendered is None:
                return  # TODO placeholder delete if placeholder.text is empty?

            if self.placeholder.placeholder_format.type == PP_PLACEHOLDER.PICTURE:
                self.render_picture(rendered)
            else:
                self.placeholder.text = rendered
        except AlreadyRenderedException:
            pass

    def render_picture(self, path):
        if os.path.exists(path):
            self.placeholder = self.placeholder.insert_picture(path)
        else:
            warnings.warn("File '{}' does not exist.")

    def insert_link(self, url, description, add_break=False):
        """
        Insert a hyperlink into the placeholder.

        It is assumed that the placeholder contains only one paragraph. If more
        paragraphs exist, the link is naively inserted into the first one.

        :param add_break: True|False: whether to put the link in the same run
          or not.

        TODO: option to add link to new paragraph.
        """
        paragraph = self.placeholder.text_frame.paragraphs[0]
        run = paragraph.add_run()
        run.text = description
        run.hyperlink.address = url
        if add_break:
            run = paragraph.add_run()
            run.text = '\n'

    @property
    def is_empty(self):
        if hasattr(self.placeholder, 'text') and self.placeholder.text:
            return False
        else:
            return self.placeholder.height == 0

    def remove(self):
        shape = self.placeholder.element
        if shape.getparent():
            shape.getparent().remove(shape)
